@@ -1,24 +1,24 @@
-## no critic (Capitalization)
 package IO::Stream::MatrixSSL::const;
-
+use 5.010001;
 use warnings;
 use strict;
+use utf8;
+use Carp;
 
-use version; our $VERSION = qv('1.0.0');
-
-# update DEPENDENCIES in POD & Makefile.PL & README
+our $VERSION = 'v2.0.0';
 
 # Timeouts:
 use constant TOHANDSHAKE    => 30;
 
 # Custom errors:
 use constant ETOHANDSHAKE   => 'ssl handshake timeout';
+use constant CERTVALIDATOR_INTERNAL_ERROR => -1;
 
 
 sub import {
     my $pkg = caller;
     no strict 'refs';
-    for my $const (qw( TOHANDSHAKE ETOHANDSHAKE )) {
+    for my $const (qw( TOHANDSHAKE ETOHANDSHAKE CERTVALIDATOR_INTERNAL_ERROR )) {
         *{"${pkg}::$const"} = \&{$const};
     }
     return;
